/**
 * Yield Task to generate service
 */
class Task.GenerateService extends YieldTask
{
	constructor()
	{
		::YieldTask.constructor("Generating Service");
		this._repeat = false;
		this._removable = true;
	}

	function _exec()
    {
        AILog.Info("Generating Services ...");
        local serv = null;
		local src_lst = null;
		local src = null, dst = null;
        foreach(cargo, val in  AICargoList()) {
            AIController.Sleep(2);
			if (!(cargo in TransAI.Drop_Point)) continue;
			local target_is_town = Cargo.HasTownEffect(cargo);
			dst = Servable.New(TransAI.Drop_Point[cargo], target_is_town);
			/* check if we have this class in manager */
			local found = TransAI.ServableMan.FindID(dst);
			if (found) {
				dst = TransAI.ServableMan.Item(found);
			} else {
				TransAI.ServableMan.New(dst);
			}
			
			local src_istown = Cargo.HasTownEffect(cargo) && Cargo.TownList_Producing(cargo).Count() > 0;
			if (src_istown) {
				/* working with town */
				src_lst = Cargo.TownList_Producing(cargo);
			} else {
				/* let's assume working with industry */
				src_lst = AIIndustryList_CargoProducing(cargo);
			}			
            local c = 0;
			local key = "";
			local id = 0;
            foreach (source, val in src_lst) {
                AIController.Sleep(1);
                src = Servable.New(source, src_istown);
                found = TransAI.ServableMan.FindID(src);
                if (found) {
                	src = TransAI.ServableMan.Item(found);
                } else {
                	TransAI.ServableMan.New(src);
                }
				key = Services.CreateID(source, dst.GetID(), cargo);
				id = TransAI.ServiceMan.FindKey(key);
				if (id) {
					AILog.Info("Found duplicate");
					serv = Services.MakeClass(TransAI.ServiceMan.Item(id));
					if (serv.Mem.Skipped)  {
						serv.Mem.Skipped = false;
						TransAI.ServiceMan.ChangeItem(id, serv.Mem.GetStorage());
					}
				} else {					
					serv = Services.New(src, dst, cargo);
					TransAI.ServiceMan.New(serv, AICargo.GetCargoIncome(cargo, 20, 200));
					c++;
					AILog.Info(AICargo.GetCargoLabel(cargo) +
					   " from " + src.GetName()+ " to " + dst.GetName());
					yield c;
				}
				serv = null;
				src = null;
            }
			//AILog.Info("Source count:" + c);
        }
        AILog.Info("Service Generator Stopped");
    }
}

/**
 * Task to generate drop off point.
 * Generate table of drop off point by industry types.
 */
class Task.GenerateDropOff extends TaskItem
{
    constructor()
    {
        ::TaskItem.constructor("Generating DropOff");
        this._removable = true;
    }

    function Execute()
    {
        AILog.Info("Generating Drop of points ... ");
        local destiny = AIList();
        foreach (cargo, val in AICargoList()) {
            AIController.Sleep(1);
            if (Cargo.HasTownEffect(cargo)) {
                destiny = AITownList();
                destiny.Valuate(Cargo.TownCanAccept, cargo);
                destiny.RemoveValue(0);
            } else {
                destiny = AIIndustryList_CargoAccepting(cargo);
            }
            if (destiny == null) continue;
            if (destiny.IsEmpty()) continue;
            TransAI.Drop_Point[cargo] <- destiny.Begin();
            destiny = null;
        }
        /* check the values
        foreach (idx, val in TransAI.Drop_Point) {
          AILog.Info(AIIndustry.GetName(val) + ":" + AICargo.GetCargoLabel(idx));
        }
        */
        AILog.Info("Drop off point Finished");
        return true;
    }
}
