/**
 * General un categorized static functions to assist program
 */
class Assist
{

	/**
	 * Count cargo that accept by an Industry
	 * @param id Industry ID
	 * @return number of cargo
	 */
	static function CargoCount(id)
	{
		local ret = 0;
		local type = AIIndustry.GetIndustryType(id);
		local cargoes = AIIndustryType.GetAcceptedCargo(type);
		if (cargoes) {
			ret += cargoes.Count();
		}
		return ret;
	}

	/**
     * Lead a number with zero
     * this will solve problem of '09' that displayed '9' only
     * @param integer_number to convert
     * @return number in string
     * @note only for number below 10
     */
    static function LeadZero(integer_number)
    {
        if (integer_number > 9) return integer_number.tostring();
        return "0" + integer_number;
    }
}

class Cargo
{
	static function HasTownEffect(cargo) {
		local te = AICargo.GetTownEffect(cargo);
		return (te == AICargo.TE_PASSENGERS ||
			te == AICargo.TE_MAIL ||
			te == AICargo.TE_GOODS ||
			te == AICargo.TE_WATER ||
			te == AICargo.TE_FOOD );
	}

	 /**
     * Check if this town can accept a cargo
     * @param id Town ID
     * @param cargo Cargo to check
     * @return True if this town can accept that cargo
     */
    static function TownCanAccept(id, cargo)
    {
        if (!AITown.IsValidTown(id)) return false;
		if (!Cargo.HasTownEffect(cargo)) return false;
        return AITile.GetCargoAcceptance(AITown.GetLocation(id), cargo, 1, 1, 5) > 6;
    }

    /**
     * Check if this town can produce a cargo
     * @param id Town ID
     * @param cargo Cargo to check
     * @return True if this town can produce that cargo
     */
    static function TownCanProduce(id, cargo)
    {
        if (!AITown.IsValidTown(id)) return false;
        return AITown.GetMaxProduction(id, cargo) != -1;
    }

	static function TownList_Producing(cargo)
	{
		local tl = AITownList();
		tl.Valuate(Cargo.TownCanProduce, cargo);
		tl.RemoveValue(0);
		return tl;
	}

	static function TownList_Accepting(cargo)
	{
		local tl = AITownList();
		tl.Valuate(Cargo.TownCanAccept, cargo);
		tl.RemoveValue(0);
		return tl;
	}
}
