 /**
 * Base Serv-able class for store either town or industry handling in game
 */
class Servable extends Storable
{
    _API = null; ///< common API function to be used by this class
    constructor()
    {
        ::Storable.constructor();
        this._storage._is_town <- true; ///< Flag of town
        this._API = AITown;
    }

    /**
     * Get the name of servable object
     * @return string name
     */
    function GetName() { return this._API.GetName(this.GetID()); }

    /**
     * Get type of servable object
     * @return true if Town or  false if Industry
     */
    function IsTown() { return this._storage._is_town; }

    /**
     * Get the location of servable object
     * @return tile index of location
     */
    function GetLocation() { return this._API.GetLocation(this.GetID()); }

    /**
     * New Servable Town
     */
    static function NewTown(id)
    {
        local tmp = Servable();
        tmp.SetID(id);
        tmp._API = AITown;
        return tmp;
    }

    /**
     * New Servable Industry
     */
    static function NewIndustry(id)
    {
        local tmp = Servable();
        tmp.SetID(id);
        tmp._API = AIIndustry;
        tmp._storage._is_town <- false; ///< Flag of town
        return tmp;
    }

    /**
     * New Unknown Servable
     */
    static function New(id, istown)
    {
        if (istown) return Servable.NewTown(id);
        return Servable.NewIndustry(id);
    }

    /**
     * Make class from storage
     * @param storage to build the class from
     */
    static function MakeClass(storage)
    {
        assert(typeof(storage) == "table");
        local tmp = Servable();
        if (!storage._is_town) tmp._API = AIIndustry;
        tmp.SetStorage(storage);
        return tmp;
    }
}

