/**
 * Fake Task Namespace
 */
class Task {}

/**
 * Task item is base for all task item
 */
class TaskItem
{
	_removable = null;
	_taskname = null;
	_result = null;

	constructor(name)
	{
		this._removable = true;
		this._taskname = name;
		this._result = null;
	}

	/**
	 * Is this task removable
	 * @return true if we should delete this task after execute it
	*/
	function IsRemovable() { return this._removable }

	/**
	 * Execute this task
	 * @note to be overriden by class descendants
	 * @return null if not execute anything
	 */
	function Execute()
	{
		AILog.Warning("Not defined Execute method yet");
	}

	/**
	 * Get the result of executing this task
	 * @return anything
	 */
	function GetResult() { return this._result; }

	/**
	 * Get the name of task
	 * @return task name
	 */
	function GetName() {return this._taskname; }
}

/**
 * Task that is using yield from generator
*/
class YieldTask extends TaskItem
{
	///last yield holder
	last_yield = null;
	/// flag to repeat generator
	_repeat = null;
	
	constructor(name)
	{
		::TaskItem.constructor(name);
		this.last_yield = "new";
		this._repeat = true;
		this._removable = false;
	}

	function Execute()
	{
		if (this.last_yield == "new") this.last_yield = this._exec();
		try {
			this._result = resume this.last_yield;
			return this._result;
		}
		catch (last_yield) {
			AILog.Info("Generator stopped");
		}
		if (this._repeat) {
			AILog.Info("Rebuild yield");
			this.last_yield = "new";
			return true;
		}
		this._result = "stop";
		
	}

	function _exec()
	{
		AILog.Warning("Not defined _exec method yet");
	}
}

